#!/usr/local/bin/ruby
# Copyright (c) 2000 Brian Fundakowski Feldman
# All rights reserved.

def mix(from, to, bias)
	bias = 1.0 - bias
	ret = from.dup
	(0..2).each {|c| ret[c] += ((to[c] - from[c]) * bias).to_i}
	return ret
end

def grad(s, from, to, step = 1)
	# begin error checking :)
	return false if !s.is_a?(String) || !from.is_a?(String) || \
	    !to.is_a?(String) || !step.is_a?(Fixnum)
	return false if from.length < 6 || to.length < 6 || step <= 0
	from = from[1..-1] if from[0] == ?#
	to = to[1..-1] if to[0] == ?#
	return false if from.length != 6 || to.length != 6
	# end error checking
	x = Array.new
	(0..2).each {|c| x[c] = to[(c * 2)..(c * 2 + 1)].hex}
	y = Array.new
	(0..2).each {|c| y[c] = from[(c * 2)..(c * 2 + 1)].hex}
	off = 0
	intag = 0
	didmix = false
	wasintag = false
	s.each_byte {|b|
		islast = off == s.length - 1
		ischange = off % step == 0
		wasintag = false
		case b
		when ?<
			intag += 1
		when ?>
			intag -= 1
			wasintag = true if intag == 0
		end
		if intag == 0 && !wasintag && (!didmix || ischange || islast)
			if didmix
				print "</font>"
			else
				didmix = true
			end
			m = mix(x, y, off.to_f / (s.length - 1))
			printf('<font color="#' + "%0.2x" * 3 + '">%c',
			    m[0], m[1], m[2], b)
			if islast
				print "</font>"
			end
		else
			printf('%c', b)
		end
		off += 1
	}
	return true
end

def usage
	print "usage: #{$0} phrase ... #fromcolor #tocolor step\n"
	exit 1
end

usage if ARGV.size < 3 

usage if grad(ARGV[0..-4].join(' '), ARGV[-3], ARGV[-2], ARGV[-1].to_i) != true

print "\n"
